package LCE.view;

import LCE.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public final class GUI extends JFrame {

    GraphicsManager graphMan;
    JMenuBar Menu;
    ToolGateBar Tool;
    JSplitMiddlePanel Split;
    JFileChooser fileChooser;
    Image iconprogram = Toolkit.getDefaultToolkit().getImage(Startup.class.getClassLoader().getResource(Startup.pathMain + "img/logo.gif"));
    //Non implementati.
    public Action loadAction;
    public Action saveAction;
    public Action cutAction;
    public Action copyAction;
    public Action pasteAction;

    public GUI() {
        //Setup GUI
        super("Logic Circuit Editor v1.0");
        setLayout(new BorderLayout());
        setSize(800, 600);
        setPreferredSize(new Dimension(400, 300));
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        setIconImage(iconprogram);


        createMenu();
        Tool = new ToolGateBar();
        graphMan = new GraphicsManager(this);
        Split = new JSplitMiddlePanel();

        initializesGraphicsManager();

        //Centra la finestra al centro dello schermo
        setLocation((Toolkit.getDefaultToolkit().getScreenSize().width - this.getWidth()) / 2, (Toolkit.getDefaultToolkit().getScreenSize().height - this.getHeight()) / 2);

        //Crea una JDialog di controllo alla chiusura.
        addWindowListener(new WindowAdapter() {

            @Override
            public void windowClosing(WindowEvent e) {
                int answer = JOptionPane.showConfirmDialog(e.getWindow(), "Do you really want to quit?", "Exit confirmation", JOptionPane.YES_NO_OPTION);
                if (answer == JOptionPane.YES_OPTION) {
                    e.getWindow().dispose();
                }
            }

            @Override
            public void windowOpened(WindowEvent e) {
                Split.MP.requestFocusInWindow();
            }
        });

        setResizable(true);
        setJMenuBar(Menu);
        add(Tool, BorderLayout.PAGE_START);
        add(Split, BorderLayout.CENTER);
        setVisible(true);
    }

    //Inizializzo i riferimenti al GraphicsManager
    public void initializesGraphicsManager() {
        graphMan.initializesSplitComponents(this);
        Split.MP.initializesRefGraphicsManager(graphMan);
        Split.SP.initializesRefGraphicsManager(graphMan);
        Tool.initializesGraphicsManager(graphMan);
        initializesGraphicsManagerInMenu();
    }

    public void initializesGraphicsManagerInMenu() {
        for (int i = 0; i < Menu.getMenuCount(); i++) {
            for (int j = 0; j < Menu.getMenu(i).getItemCount(); j++) {
                if (Menu.getMenu(i).getItem(j) instanceof JMenuItemPAUSE) {
                    ((JMenuItemPAUSE) Menu.getMenu(i).getItem(j)).initializesRefGraphicsManager(graphMan);
                }
                if (Menu.getMenu(i).getItem(j) instanceof JMenuItemPLAY) {
                    ((JMenuItemPLAY) Menu.getMenu(i).getItem(j)).initializesRefGraphicsManager(graphMan);
                }
                if (Menu.getMenu(i).getItem(j) instanceof JMenuItemRESIZE) {
                    ((JMenuItemRESIZE) Menu.getMenu(i).getItem(j)).initializesRefGraphicsManager(graphMan);
                }
            }
        }
    }

    protected void createMenu() {
        Menu = new JMenuBar();
        JMenu File = new JMenu("File");
        JMenuItem Nuovo = new JMenuItem("New");
        JMenuItem Esci = new JMenuItem("Exit");
        JMenu Modifica = new JMenu("Component");
        JMenuItem Cancella = new JMenuItem("Delete");
        JMenuItem IsolaPorta = new JMenuItem("Disconnect");
        JMenu Simulazione = new JMenu("Simulation");
        JMenuItemPAUSE Pause = new JMenuItemPAUSE("Pause");
        JMenuItemPLAY Play = new JMenuItemPLAY("Play", Pause);
        Pause.setPlay(Play);
        JMenuItemRESIZE Resize = new JMenuItemRESIZE("Resize Workspace");



        Esci.addActionListener(new AbstractAction("Esci") {

            @Override
            public void actionPerformed(ActionEvent e) {
                int answer = JOptionPane.showConfirmDialog(Startup.window, "Do you really want to quit?", "Exit confirmation", JOptionPane.YES_NO_OPTION);
                if (answer == JOptionPane.YES_OPTION) {
                    Startup.window.dispose();
                }
            }
        });

        Nuovo.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                graphMan.resetCircuit();
            }
        });
        
        IsolaPorta.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                graphMan.isolateJLinkablePanel(graphMan.getSelectedJLinkablePanel(), graphMan.searchJLinkablPanel(graphMan.selectedComp));
            }
        });

        Cancella.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                graphMan.searchDeleteSelected();
            }
        });


        File.add(Nuovo);
        File.add(Resize);
        File.addSeparator();
        File.add(Esci);

        Modifica.add(IsolaPorta);
        Modifica.add(Cancella);
        Modifica.setEnabled(false);

        Simulazione.add(Play);
        Simulazione.add(Pause);
        Pause.setEnabled(false);

        Menu.add(File);
        Menu.add(Modifica);
        Menu.add(Simulazione);
    }
}
