package LCE.model;

import LCE.*;
import LCE.controller.*;
import LCE.view.*;

import java.util.*;

public class Circuit 
{
    ArrayList<Piece> LogicCircuit; 	//Contiene le porte che formano il circuito
    ArrayList<Link> Line;			//Contiene i collegamenti
    ArrayList<Input> InList;		//Contiene gli input
    ArrayList<Led> OutList;			//Contiene gli output

    //Crea le liste da usare
    public Circuit() 
	{
        LogicCircuit = new ArrayList<Piece>();
        Line = new ArrayList<Link>();
        InList = new ArrayList<Input>();
        OutList = new ArrayList<Led>();
    }

    //METODI SUI PieceI
    //Cerca la porta attraverso l'ID
    public Piece searchPiece(int ID) 
	{
        for (int i = 0; i < LogicCircuit.size(); i++) 
		{
            if (LogicCircuit.get(i).getID() == ID) 
			{
                return LogicCircuit.get(i);
            }
        }
        return null;
    }

	//Metodo inutilizzato
    public int indexOfPiece(Piece c) 
	{
        return LogicCircuit.indexOf(c);
    }

    //Inserisci una porta del tipo specificato nel livello specificato
    public int insertPiece(Piece c) 
	{
        if (c != null) 
		{
            LogicCircuit.add(c); 
			return c.getID();
        }
		System.out.println("Errore nell'inserimento di un componente");
        return -1;   
    }

    //Inserisce il Piecee, creandolo prima in base ai parametri
    public int insertPiece(int tipo, int x, int y) 
	{
        Piece c = createPiece(tipo, x, y);
        insertPiece(c);
        return c.getID();
    }

    //Cancella porta passata
    public void deletePiece(Piece c) 
	{
        LogicCircuit.remove(c);
    }

    //Crea il Piecee in base al tipo, settando anche le coordinate
    public Piece createPiece(int tipo, int x, int y) 
	{
        switch (tipo) 
		{
            case Data.AND_GATE:
                return new AndGate(x, y);
            case Data.NAND_GATE:
                return new NAndGate(x, y);
            case Data.OR_GATE:
                return new OrGate(x, y);
            case Data.NOR_GATE:
                return new NOrGate(x, y);
            case Data.NOT_GATE:
                return new NotGate(x, y);
            case Data.XOR_GATE:
                return new XOrGate(x, y);
            case Data.NXOR_GATE:
                return new NXOrGate(x, y);
            case Data.POINT:
                return new Point(x, y);
            default:
				System.out.println("Errore creazione pezzo");
                return null;
        }
    }

    public int typeOfGate(int idPiece) 
	{
        Piece p = searchPiece(idPiece);
        if (p instanceof AndGate) {
            return Data.AND_GATE;
        }else
        if (p instanceof NAndGate) {
            return Data.NAND_GATE;
        }else
        if (p instanceof OrGate) {
            return Data.OR_GATE;
        }else
        if (p instanceof NOrGate) {
            return Data.NOR_GATE;
        }else
        if (p instanceof XOrGate) {
            return Data.XOR_GATE;
        }else
        if (p instanceof NXOrGate) {
            return Data.NXOR_GATE;
        }else
        if (p instanceof NotGate) {
            return Data.NOT_GATE;
        }else
        if (p instanceof Point) {
            return Data.POINT;
        }
		System.out.println("Errore nel typeOfGate");
        return -1;
    }

    //Elimina la porta in base alle coordinate
    public void deletePiece(int ID) 
	{
        LogicCircuit.remove(searchPiece(ID));
    }

    //Cerca la porta confrontandone le coordinate con i parametri
    public Piece searchPiece(int _x, int _y) 
	{
        for (int i = 0; i < LogicCircuit.size(); i++) 
		{
            if (LogicCircuit.get(i).x == _x && LogicCircuit.get(i).y == _y) 
			{
                return LogicCircuit.get(i);
            }
        }
        return null;
    }

    public void setInputOfGate(int idIn, boolean value, int idGate) 
	{
        Piece t = searchPiece(idGate);
        if (t instanceof Gate) 
		{
            ((Gate) t).setInput(idIn, value);
        }
    }

    public boolean getInputOfPiece(int idPiece, int idIn) 
	{
        Piece t = searchPiece(idPiece);
        if (t == null) {
            t = searchInput(idPiece);
        }
        if (t == null) {
            t = searchOutput(idPiece);
        }
        if (t == null) {
            return false;
        }
        return t.getInput(idIn);
    }

    public boolean getOutputOfPiece(int idPiece, int idIn) 
	{
        Piece t = searchPiece(idPiece);
        if (t == null) {
            t = searchInput(idPiece);
        }
        if (t == null) {
            t = searchOutput(idPiece);
        }
        if (t == null) {
            return false;
        }
        return t.getOutput();
    }

    //METODI SUI COLLEGAMENTI
    //Inserisci il collegamento specificato
    public void insertLink(Link t) {
        Line.add(t);
    }

    //Inserisce il collegamento tra le porte specificate
    public int insertLink(Piece in, Piece out, int idIn, int idOut) 
	{
		if(out == null){
			System.out.println("Errore in insertLink2");
			return -1;
		}
		if( in instanceof Input && ((Input) in).collegamento != null ||
			in instanceof Gate && ((Gate) in).collegamento != null  ||
			!chkInputGate(out, idOut)
		){
			return -1;
		}
		Link t = new Link(in, out, idIn, idOut);
		this.insertLink(t);
		return t.getID();
    }

    //Inserisce il collgemento con gli id delle Piecei e dell'ingresso
    public int insertLink(int in, int out, int idIn, int idOut) {
        Piece inPiece = searchPiece(in);
        if (inPiece == null) {
            inPiece = searchInput(in);
        }
		if(inPiece == null) System.out.println("Errore in insertLink: Circuit riga 217 circa");
		
        Piece outPiece = searchPiece(out);
        if (outPiece == null) {
            outPiece = searchOutput(out);
        }

        return insertLink(inPiece, outPiece, idIn, idOut);
    }

    //Cerca il link in base all'ID
    public Link searchLink(int idLink) 
	{
        for (int i = 0; i < Line.size(); i++) 
		{
            if (Line.get(i).getID() == idLink) 
			{
                return Line.get(i);
            }
        }
        return null;
    }

    //Restituisce la porta in ingresso tramite ID
    public int getInputLink(Link t) {
        for (int i = 0; i < LogicCircuit.size(); i++) {
            if ((LogicCircuit.get(i) instanceof Point && ((Point) LogicCircuit.get(i)).contain(t))
                    || (LogicCircuit.get(i) instanceof Gate && ((Gate) LogicCircuit.get(i)).collegamento == t)) {
                return LogicCircuit.get(i).getID();
            }
        }
        for (int i = 0; i < InList.size(); i++) {
            if (InList.get(i).collegamento == t) {
                return InList.get(i).getID();
            }
        }
        return -1;
    }

    public int getInputLink(int idLink) {
        return getInputLink(searchLink(idLink));
    }

    //Restituisce la Piecee in uscita
    public int getOutputLink(Link t) {
        return t.getOutput().getID();
    }

    public int getOutputLink(int idLink) {
        return getOutputLink(searchLink(idLink));
    }

    //Elimina il collegamento specificato
    public void deleteLink(int idLink) 
	{
        deleteLink(searchLink(idLink));
    }

    public void deleteLink(Link t) 
	{
        Line.remove(t);
        for (int i = 0; i < LogicCircuit.size(); i++) 
		{	
            if (LogicCircuit.get(i) instanceof Point && ((Point) LogicCircuit.get(i)).contain(t)) 
			{
                ((Point) LogicCircuit.get(i)).deleteLink(t);
            }
			
            if (LogicCircuit.get(i) instanceof Gate && ((Gate) LogicCircuit.get(i)).collegamento == t) 
			{
                ((Gate) LogicCircuit.get(i)).collegamento = null;
            }
			
            if (t.getOutput() == LogicCircuit.get(i)) 
			{
                LogicCircuit.get(i).setInput(t.idIn, false);
            }
        }
		
        for (int i = 0; i < InList.size(); i++) 
		{
            if (InList.get(i).collegamento == t) 
			{
                InList.get(i).collegamento = null;
            }
        }
    }

    protected boolean chkInputGate(Piece p, int idPed) {
        int i = 0;
        while (i < Line.size()) {
            if (Line.get(i).getOutput() == p && Line.get(i).idIn == idPed) {
                return false;
            }
            i++;
        }
        return true;
    }

    //METODI SUL CIRCUITO
    //Isola la porta specificata tramite l'ID
    public void isolateGate(int ID) 
	{
		//TANO: Da controllare
        ArrayList<Link> t = new ArrayList<Link>();
        Piece t1 = searchPiece(ID);
        if (t1 instanceof Gate || t1 instanceof Point) {
			for (int i = 0; i < Line.size(); i++) {
				if(
					(	t1 instanceof Gate && 
						(	Line.get(i) == ((Gate) t1).collegamento || 
							Line.get(i).getOutput() == t1
						)
					) || (
						t1 instanceof Point && 
						(	((Point) (t1)).contain(Line.get(i)) || 
							Line.get(i).getOutput() == t1
						)
					)
				){
					t.add(Line.get(i));
				}
			}
			for (int i = 0; i < t.size(); i++) 
			{
				deleteLink(t.get(i));
			}
		}
    }

    public Input searchInput(int idInput) 
	{
        for (int i = 0; i < InList.size(); i++) 
		{
            if (InList.get(i).getID() == idInput) {
                return InList.get(i);
            }
        }
        return null;
    }

    public void isolateInput(int ID) 
	{
        if (!(searchInput(ID) instanceof Input)) {
            return;
        }
        Input tmp = (Input) searchInput(ID);
        for (int i = 0; i < Line.size(); i++) {
            if (tmp.collegamento == Line.get(i)) {
                deleteLink(Line.get(i));
            }
        }
    }

    //Restituisce il numero di input
    public int numInput() {
        return InList.size();
    }

    //Restituisce la lista di input
    public ArrayList<Input> getInput() {
        return InList;
    }

    //Setta un input
    public void setInput(int idInput, boolean value) {
        if (searchInput(idInput) == null) {
            return;
        }
        searchInput(idInput).setInput(0, value);
    }

    //Restituisce il numero di output
    public int numOutput() {
        return OutList.size();
    }

    //Restituisce la lista di output
    public ArrayList<Led> getOutput() {
        return OutList;
    }

    //Aggiunge un input
    public int addInput() {
        Input t = new Input(0, 0);
        InList.add(t);
        return t.getID();
    }

    //Elimina un input
    public void deleteInput(int idInput) {
        Input t = searchInput(idInput);
        isolateInput(idInput);
        InList.remove(t);
    }

    public Led searchOutput(int idOutput) 
	{
        for (int i = 0; i < OutList.size(); i++) {
            if (OutList.get(i).getID() == idOutput) {
                return OutList.get(i);
            }
        }
        return null;
    }

    public void isolateOutput(int ID) 
	{
        if ((searchOutput(ID) instanceof Led)) 
		{
			Led tmp = (Led) searchOutput(ID);
			for (int i = 0; i < Line.size(); i++) 
			{
				if (Line.get(i).getOutput() == tmp) 
				{
					deleteLink(Line.get(i));
				}
			}
		}
    }

    //Aggiunge un output
    public int addOutput() {
        Led t = new Led(0, 0);
        OutList.add(t);
        return t.getID();
    }

    //Toglie un output
    public void deleteOutput(int idLed) 
	{
        Led t = searchOutput(idLed);
        isolateOutput(idLed);
        OutList.remove(t);
    }

//    //Aggiorna i valori del circuito
//    public void refreshCircuit() {
//        for (int i = 0; i < InList.size(); i++) {
//            InList.get(i).refreshValue();
//        }
//    }
    public void refreshCircuit() 
	{
        for (int i = 0; i < InList.size(); i++) {
            InList.get(i).refreshValue();
        }
        for (int i = 0; i < LogicCircuit.size(); i++) {
            LogicCircuit.get(i).refreshValue();
        }
        for (int i = 0; i < OutList.size(); i++) {
            OutList.get(i).refreshValue();
        }
    }

    public String StampaPieces() {
        ArrayList<Piece> t = new ArrayList<Piece>();
        t.addAll(LogicCircuit);
        t.addAll(InList);
        t.addAll(OutList);
		String tmp = "";
        for (int i = 0; i < t.size(); i++) {
            tmp+=t.get(i).toString()+'\n';
        }
		return tmp;
    }

    public String StampaLine() 
	{
		String tmp = "";
        for (int i = 0; i < Line.size(); i++) {
            tmp+=Line.get(i).toString()+'\n';
        }
		return tmp;
    }

    public void setLed(int idLed, boolean value) {
        Led t = searchOutput(idLed);
        if (t == null) {
            return;
        }
        t.setInput(0, value);
    }

    public void setGatePosition(int idPiece, int x, int y) {
        Piece t = searchPiece(idPiece);
        if (t == null || !(t instanceof Gate)) {
            return;
        }
        t.movePiece(x, y);
    }

}